﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace RandomProblemsFramework
{
    class Program
    {
        /// <summary>
        /// In this example we have shared object of Random class.
        /// There is a problem with concurrent using of Next() methods
        /// </summary>
        public static void GlobalRandom()
        {
            Random globalRand = new Random();
            List<Thread> threads = new List<Thread>();
            for (int i = 0; i < 100; i++)
            {
                Thread th = new Thread(() =>
                {
                    int lastRandomValue = -1;
                    int currentRandomValue = -1;
                    int repeatCounter = 0;
                    for (int ii = 0; ii < 1000; ii++)
                    {
                        //lock(globalRand) //uncoment to avoid spoiling random generator
                        currentRandomValue = globalRand.Next();
                        if (lastRandomValue == currentRandomValue) //when previous value is equal to current one we remember that
                        {
                            //  Console.WriteLine($"currentRandomValue is equal to the previous one: {currentRandomValue}");
                            repeatCounter++;
                        }
                        lastRandomValue = currentRandomValue;
                    }
                    Console.WriteLine($"repeatCounter {repeatCounter}"); //after all we print numbers of repetitions
                }
                );
                threads.Add(th);
                th.Start();
            }
            threads.ForEach(th => th.Join());
        }


        static readonly int threadsNo = 25;
        static readonly int iterationsNo = 10;

        static int[] values = new int[threadsNo]; //the place where threads can store their values
        //Barrier to synchronise each iteration and print array of values
        static Barrier barrier = new Barrier(threadsNo, (b) => {
            foreach (var v in values)
            {
                Console.Write($"[{v}]");
            }
            Console.WriteLine();
        }
        );

     
        /// <summary>
        /// Example of using Default Local Random
        /// </summary>
        public static void DefaultLocalRandom()
        {
            List<Thread> threads = new List<Thread>();
            for (int i = 0; i < threadsNo; i++)
            {
                Thread th = new Thread((o) =>
                {
                    int threadIndex = (int)o;
                    Random localRandom = new Random(); //look at the .net core source code https://github.com/dotnet/corert/blob/master/src/System.Private.CoreLib/shared/System/Random.cs 
                    //Random localRandom = new Random(Thread.CurrentThread.ManagedThreadId);
                    //Random localRandom = new Random(Thread.CurrentThread.ManagedThreadId + (int)DateTime.Now.Ticks);
                    for (int ii = 0; ii < iterationsNo; ii++)
                    {
                        int randomValue = localRandom.Next(10);
                        values[threadIndex] = randomValue; //fill with random value
                        barrier.SignalAndWait(); //synchronize each repetition
                    }
                }
                );
                threads.Add(th);
            }
            int thIdx = 0;
            threads.ForEach(th => th.Start(thIdx++)); //Start all the threads passing them index no.
            threads.ForEach(th => th.Join()); //wait for all threads.
        }

        static void Main(string[] args)
        {
            //GlobalRandom();
            DefaultLocalRandom();
        }
    }
}
